@@ -1,6 +1,6 @@
 /**
  *	This file is part of devilspie2
- *	Copyright (C) 2011-2015 Andreas Rönnquist
+ *	Copyright (C) 2011-2017 Andreas Rönnquist
  *
  *	devilspie2 is free software: you can redistribute it and/or
  *	modify it under the terms of the GNU General Public License as published
@@ -981,7 +981,7 @@ int c_unstick_window(lua_State *lua)
 
 
 /**
- * return the geometry of the current window to the LUA script
+ * return the geometry of the current window to the Lua script
  */
 int c_get_window_geometry(lua_State *lua)
 {
@@ -1010,7 +1010,7 @@ int c_get_window_geometry(lua_State *lua)
 
 
 /**
- * return the client geometry of the current window to the LUA script
+ * return the client geometry of the current window to the Lua script
  * this is excluding the window manager frame
  */
 int c_get_client_window_geometry(lua_State *lua)
@@ -1578,63 +1578,121 @@ int c_set_window_fullscreen(lua_State *lua)
 int c_set_viewport(lua_State *lua)
 {
 	int top = lua_gettop(lua);
-	WnckScreen *screen;
-	int x,y,width,height, viewport_start;
+	int width, height;
+	int viewport_start_x, viewport_start_y;
+	int win_x, win_y;
+	
+	if (top == 1) {
 
-	if (top != 1) {
-		luaL_error(lua, "set_viewport: %s", one_indata_expected_error);
+		WnckScreen *screen;
+		int x;
+		int type = lua_type(lua, 1);
+		if (type != LUA_TNUMBER) {
+			luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
+			return 0;
+		}
+
+		int num = lua_tonumber(lua,1);
+
+		if (num <= 0) {
+			g_error("set_viewport: %s", integer_greater_than_zero_expected_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		WnckWindow *window = get_current_window();
+
+		if (!window) {
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		screen = wnck_window_get_screen(window);
+
+		wnck_window_get_geometry(window, &win_x, &win_y, &width, &height);
+
+		gulong xid = wnck_window_get_xid(window);
+
+		//viewport_start = devilspie2_get_viewport_start(xid);
+		if (devilspie2_get_viewport_start(xid, &viewport_start_x, &viewport_start_y) != 0) {
+			g_printerr("set_viewport: %s", could_not_find_current_viewport_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		x = ((num - 1) * wnck_screen_get_width(screen)) - viewport_start_x + win_x;
+		
+		devilspie2_error_trap_push();
+		XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
+								wnck_window_get_xid(window),
+								x, win_y, width, height);
+
+		if (devilspie2_error_trap_pop()) {
+			g_printerr("set_viewport: %s", setting_viewport_failed_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		lua_pushboolean(lua, TRUE);
+		return 1;
+
+	} else if (top == 2) {
+		
+		int type1 = lua_type(lua, 1);
+		int type2 = lua_type(lua, 2);
+
+		if (type1 != LUA_TNUMBER) {
+			luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
+			return 0;
+		}
+
+		if (type2 != LUA_TNUMBER) {
+			luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
+			return 0;
+		}
+
+		int new_xpos = lua_tonumber(lua, 1);
+		int new_ypos = lua_tonumber(lua, 2);
+
+		WnckWindow *window = get_current_window();
+
+		if (!window) {
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		wnck_window_get_geometry(window, &win_x, &win_y, &width, &height);
+
+		gulong xid = wnck_window_get_xid(window);
+
+		//viewport_start = devilspie2_get_viewport_start(xid);
+		if (devilspie2_get_viewport_start(xid, &viewport_start_x, &viewport_start_y) != 0) {
+			g_printerr("set_viewport: %s", could_not_find_current_viewport_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+		
+		devilspie2_error_trap_push();
+		XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
+								wnck_window_get_xid(window),
+								new_xpos, new_ypos, width, height);
+
+		if (devilspie2_error_trap_pop()) {
+			g_printerr("set_viewport: %s", setting_viewport_failed_error);
+			lua_pushboolean(lua, FALSE);
+			return 1;
+		}
+
+		lua_pushboolean(lua, TRUE);
+		return 1;
+		
+	
+	} else {
+		luaL_error(lua, "set_viewport: %s", one_or_two_indata_expected_error);
 		return 0;
 	}
-
-	int type = lua_type(lua, 1);
-	if (type != LUA_TNUMBER) {
-		luaL_error(lua, "set_viewport: %s", number_expected_as_indata_error);
-		return 0;
-	}
-
-	int num = lua_tonumber(lua,1);
-
-	if (num <= 0) {
-		g_error("set_viewport: %s", integer_greater_than_zero_expected_error);
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	WnckWindow *window = get_current_window();
-
-	if (!window) {
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	screen = wnck_window_get_screen(window);
-
-	wnck_window_get_geometry(window, &x, &y, &width, &height);
-
-	gulong xid = wnck_window_get_xid(window);
-
-	viewport_start = devilspie2_get_viewport_start(xid);
-	if (viewport_start < 0) {
-		g_printerr("set_viewport: %s", could_not_find_current_viewport_error);
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	x=((num-1) * wnck_screen_get_width(screen)) - viewport_start + x;
-
-	devilspie2_error_trap_push();
-	XMoveResizeWindow(gdk_x11_get_default_xdisplay(),
-	                  wnck_window_get_xid(window),
-	                  x, y, width, height);
-
-	if (devilspie2_error_trap_pop()) {
-		g_printerr("set_viewport: %s", setting_viewport_failed_error);
-		lua_pushboolean(lua, FALSE);
-		return 1;
-	}
-
-	lua_pushboolean(lua, TRUE);
-	return 1;
+	
+	return 0;
 }
 
 
@@ -1765,7 +1823,7 @@ int c_set_window_type(lua_State *lua)
 
 
 /**
- * return the geometry of the screen  to the LUA script
+ * return the geometry of the screen  to the Lua script
  */
 int c_get_screen_geometry(lua_State *lua)
 {
@@ -1979,6 +2037,59 @@ int c_xywh(lua_State *lua)
 	return 0;
 }
 
+struct lua_callback {
+	lua_State *lua;
+	int ref;
+};
+
+static void on_geometry_changed(WnckWindow *window, struct lua_callback *callback)
+{
+	if (callback == NULL)
+		return;
+
+	WnckWindow *old_window = get_current_window();
+	set_current_window(window);
+
+	lua_rawgeti(callback->lua, LUA_REGISTRYINDEX, callback->ref);
+	lua_pcall(callback->lua, 0, 0, 0);
+
+	set_current_window(old_window);
+}
+
+static void on_geometry_changed_disconnect(gpointer data, GClosure *closure)
+{
+	g_free(data);
+}
+
+/**
+ *
+ */
+int c_on_geometry_changed(lua_State *lua)
+{
+	int top = lua_gettop(lua);
+	if (top != 1) {
+		luaL_error(lua, "on_geometry_changed: %s", one_indata_expected_error);
+		return 0;
+	}
+
+	if (lua_type(lua, 1) != LUA_TFUNCTION) {
+		luaL_error(lua, "on_geometry_changed: %s", "function expected");
+		return 0;
+	}
+
+	struct lua_callback *cb = g_malloc(sizeof(struct lua_callback));
+	cb->lua = lua;
+	cb->ref = luaL_ref(lua, LUA_REGISTRYINDEX);
+
+	WnckWindow *window = get_current_window();
+
+	if (window) {
+		g_signal_connect_data(window, "geometry-changed", G_CALLBACK(on_geometry_changed), (gpointer)cb, (GClosureNotify)(on_geometry_changed_disconnect), 0);
+	}
+
+	return 0;
+}
+
 /*
  * Devilspie:
 
